/*

(C) 2009-2011 Mika Ilmaranta <ilmis@nullnet.fi>

License: GPLv2

*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdarg.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "config.h"
#include "forkexec.h"

void makeplist(char *plist[], char *buf);

void forkexec(char *fmt, ...) {
	pid_t pid;
	va_list vl;
	char buf[BUFSIZ];
	char *argv[64];
	int exitval;
	char *envp[64];
	char envbuf[BUFSIZ];
#if defined(DEBUG)
	int i;
#endif

	/* fill in buf */
	va_start(vl, fmt);
	vsprintf(buf, fmt, vl);
	va_end(vl);

	/* fill in envbuf */
	sprintf(envbuf, "LANG=%s PATH=%s TERM=%s", getenv("LANG"), getenv("PATH"), getenv("TERM"));

	pid = fork();

	if(pid == -1) {
		syslog(LOG_ERR, "forkexec: fork() failed \"%s\"", strerror(errno));
		return;
	}

	if(pid) {
		/* parent */
		wait(&exitval);
		if(cfg.debug >= 9 && exitval) syslog(LOG_ERR, "script exited with non null exit value %d", exitval);
		return;
	}

	/* child */
	closelog(); /* openlog doesn't return a fd to set close on exec */

	/* make argv pointers */
	makeplist(argv, buf);

	/* make envp pointers */
	makeplist(envp, envbuf);

#if defined(DEBUG)
	for(i = 0; argv[i]; i++) {
		fprintf(stderr, "argv[%d] = \"%s\"\n", i, argv[i]);
	}
	for(i = 0; envp[i]; i++) {
		fprintf(stderr, "envp[%d] = \"%s\"\n", i, envp[i]);
	}
#endif

	execve(argv[0], argv, envp);
	exit(1); /* exec failed ... */
}

void makeplist(char *plist[], char *buf)
{
	int i,ap;

	for(i = ap = 0; buf[i]; ap++) {
		plist[ap] = buf + i;
		for(; buf[i] && buf[i] != ' '; i++);
		if(buf[i]) buf[i++] = '\0';
	}
	plist[ap] = NULL;
}

/* EOF */
